require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')
  require_relative '../../swagger_helper'

  ACCEPTABLE_PARAMS = %w[
    nud
    sne_profile_creation_date
    ss_id
    first_name
    last_name
    full_name
    patronymic_name
    nationality
    email
    email_advances_assistant
    event_notification
    civility
    birth_date
    work_contract_type
    profession_details
    has_other_employers
    main_employer_name
    main_employer_siret
    workplace_postal_code
    workplace_locality
    foreign_workplace_postal_code
    foreign_workplace_locality
    foreign_workplace_country
    year_minus_one
    amount_year_minus_one
    year_minus_two
    amount_year_minus_two
    ressources_details
    tax_revenues
    apl_amount
    postal_details
    actual_housing_details
    family_situation
    family_situation_validated
    asked_children_nb
    valid_children_nb
    asked_family_size
    valid_family_size
    situation_tax_treshold
    status
    updated_status_at
    asked_locataire_hlm
    valid_locataire_hlm
    valid_priority_public
    asked_priority_public
    single_parent
    priority_public
    mobility
    mutation
    completed_requests_nb
    on_going_request_nb
    disability
    renewal_date
    eligibility_query
    active_housing_request_id
    housing_request_ids
    entity_id
    application_ids
    monthly_incomes
    monthly_charges
    cellphone
    sms_notification
    homephone
    workphone
    main_user_id
  ]

  MUTATE_PARAMS = {
    nud: 'String',
    sne_profile_creation_date: 'DateTime',
    ss_id: 'String',
    first_name: 'String',
    last_name: 'String',
    patronymic_name: 'String',
    nationality: 'String',
    email: 'String',
    email_advances_assistant: 'String',
    event_notification: 'Boolean',
    civility: 'String',
    birth_date: 'DateTime',
    work_contract_type: 'String',
    profession_details: 'Hash',
    has_other_employers: 'Boolean',
    main_employer_name: 'String',
    main_employer_siret: 'String',
    workplace_postal_code: 'String',
    workplace_locality: 'String',
    foreign_workplace_postal_code: 'String',
    foreign_workplace_locality: 'String',
    foreign_workplace_country: 'String',
    year_minus_one: 'String',
    amount_year_minus_one: 'Float',
    year_minus_two: 'String',
    amount_year_minus_two: 'Float',
    ressources_details: 'Hash',
    tax_revenues: 'Float',
    apl_amount: 'Float',
    postal_details: 'Hash',
    actual_housing_details: 'Hash',
    family_situation: 'String',
    family_situation_validated: 'Boolean',
    situation_tax_treshold: 'Integer',
    status: 'String',
    asked_locataire_hlm: 'Boolean',
    valid_locataire_hlm: 'Boolean',
    valid_priority_public: 'Boolean',
    asked_priority_public: 'Boolean',
    priority_public: 'Boolean',
    mobility: 'Boolean',
    mutation: 'Boolean',
    disability: 'String',
    renewal_date: 'DateTime',
    monthly_incomes: 'Hash',
    monthly_charges: 'Hash',
    cellphone: 'String',
    is_owner: 'Boolean',
    'sms_notification': 'Boolean',
    'homephone': "String",
    'workphone': 'String',
    main_user_id: 'String',
    workplace_insee_code: "String"
  }

  OFFERS_ACCEPTABLE_PARAMS = %w[
    created_at
    updated_at
    eligibility_query
    typology
    lessor_status
    type
    age
    district
    department
    publication_end_date
    rent_amount
    surface
    rooms
    exclusivity_end_date
    nb_of_eligible_employees
    nb_of_requests_from_employees
    availability_date
    description
    details
    old_status
    status
    status_updated_at
    effort_rate
    min_effort_treshold
    single_parent
    reserved
    priority_public
    mobility
    bedrooms
    bathrooms
    heating_type
    floors
    has_elevator
    has_gardian
    dpe
    outside
    has_cellar
    parking_type
    pmr
    kitchen_type
    address
    postal_code
    city
    country
    coordinates
    latitude
    longitude
    mutation
    eligibility_string
    applicant_category_1
    threshold_type
    threshold_subtype
    threshold_category_2
    threshold_category_3
    threshold_category_4
    threshold_category_5
    threshold_category_6
    threshold_category_7
    threshold_category_8
    threshold_category_9
    threshold_category_10
    threshold_category_11
    threshold_category_12
    individual_heating
    date_publication_start
    date_publication_end
    lessor_id
    kind
    building_type
    allready_rent
    guarantee_deposit
    rental_charges
    car_park_rent
    offer_status
    offer_status_updated_at
    lessor_status_updated_at
    ceil_category_1
    ceil_category_2
    ceil_category_3
    ceil_category_4
    ceil_category_5
    ceil_category_6
    ceil_category_7
    ceil_category_8
    ceil_category_9
    ceil_category_10
    ceil_category_11
    ceil_category_12
    floor_category_1
    floor_category_2
    floor_category_3
    floor_category_4
    floor_category_5
    floor_category_6
    floor_category_7
    floor_category_8
    floor_category_9
    floor_category_10
    floor_category_11
    floor_category_12
    min_revenues
    total_charges
    max_effort_rate
    charges
    tense_area
    subsidiary_company
    rpls_id
    operation_name
    administrator
    administration_area
    potential_request_name_town
    potential_request_name_county
    number_of_companies_30_minutes_around
    construction_year
    address_can_be_shared
    qpv
    total_floors
    address_extended
    dpe_gas
    test_created_at
    new_offer
    lessor_name
    lessor_siren
    als_siren
    postal_code
    residence_code
    pictures
    agency_id
    agency_name
    lessor_contact_name
    lessor_email
    lessor_phone_number
    rent_without_charges
    garden
    green_area
    condo
    furnished_lodging
    virtual_visit
    duplex
    first_quartile
    other_quartiles
    lessor_convention_reference
    reserving_agency_name
    reserving_agency_siren
    is_owner
  ]

  VALID_OPERATORS = %w[
    $in
    $nin
    $ne
    $gt
    $gte
    $lt
    $lte
    $eq
    $regex
    $all
  ]

  describe Api::Dmo::ApplicantsController, :type => :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    context 'with project oriented roles' do
      let!(:demandeur) { create(:user, auth_roles: ['Demandeur']) }

      let!(:bailleur) { create(:user, auth_roles: ['Bailleur']) }
      let!(:entreprise) { create(:user, auth_roles: ['Entreprise']) }

      describe 'POST as entreprise with main_user_id' do
        before :each do
          authenticated_header(request, demandeur)
          post 'create', {
            data: {
              attributes: {
                main_user_id: demandeur.id.to_s
              }
            }
          }
        end

        it 'returns status code 201' do
          expect(response).to have_http_status(201)
        end

        it 'returns valid json' do
          attributes = json['data']['attributes']
          expect(attributes['main_user_id']).to eq demandeur.id.to_s
          expect(demandeur.reload.main_applicant_id.to_s).to eq json['data']['id']
        end
      end

      describe 'POST as entreprise' do
        before :each do
          authenticated_header(request, entreprise)
          post 'create', {
            data: {
              attributes: {
              }
            }
          }
        end

        it 'returns status code 201' do
          expect(response).to have_http_status(201)
        end

        it 'returns valid json' do
          attributes = json['data']['attributes']
          expect(attributes['main_user_id']).to eq nil
          expect(demandeur.reload.main_applicant_id).to eq nil
        end
      end
    end

    context 'normal' do
      let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
      let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}

      describe 'POST /applicants' do
        describe 'applicant created with identity' do
          before :each do
            authenticated_header(request, responsible)
            post 'create', {
              data: {
                attributes: {
                  first_name: 'Joseph',
                  last_name: 'Joestar',
                  email: 'josephjoestar@test.fr'
                }
              }
            }
          end

          it 'returns status code 201' do
            expect(response).to have_http_status(201)
          end

          it 'returns valid JSON' do
            attributes = json['data']['attributes']
            expect(attributes['full_name']).to eq 'Joseph Joestar'
            expect(attributes['email']).to eq 'josephjoestar@test.fr'
            expect(User.where(main_applicant_id: json['data']['id']).count).to be 0
          end
        end

        describe 'Create applicant with minimal valid datas' do
          before :each do
            authenticated_header(request, responsible)
            post 'create', {
              data: {
                attributes: {
                  main_user_id: responsible.id,
                }
              }
            }
          end
          it 'returns status code 201' do
            expect(response).to have_http_status(201)
          end

          it 'returns valid JSON' do
            attributes = json['data']['attributes']
            expect(attributes['full_name']).to eq responsible.full_name
            expect(attributes['email']).to eq responsible.email
            expect(User.where(main_applicant_id: json['data']['id']).count).to be 1
          end

          describe 'then same user creates another applicant' do
            before :each do
              authenticated_header(request, responsible)
              post 'create', {
                data: {
                  attributes: {
                    first_name: 'Janine',
                    last_name: 'Joseph',
                    email: 'janinejoseph@test.fr'
                  }
                }
              }
            end

            it 'returns status code 201' do
              expect(response).to have_http_status(201)
            end

            it 'returns valid JSON' do
              attributes = json['data']['attributes']
              expect(attributes['full_name']).to eq 'Janine Joseph'
              expect(attributes['email']).to eq 'janinejoseph@test.fr'
              expect(User.where(main_applicant_id: { :$ne => nil }).count).to be 1
            end
          end
        end
        MUTATE_PARAMS.each do |param, type|
          describe 'post request with ' + param.to_s + ' ' + type.to_s + 'is valid' do
            let(:params) do
              attr = {}
              attr[param] = get_typed_value(type)
              attr
            end
            before :each do
              authenticated_header(request, responsible)
              post 'create', {
                  data: {
                      attributes: params
                  }
              }
            end
            it 'returns status code 201 and the value appears in json' do
              expect(response).to have_http_status(201)
              if (type == 'Date' || type == 'DateTime')
                expect(json['data']['attributes'][param.to_s]).not_to be_empty
                expect(json['data']['attributes'][param.to_s]).not_to eq('')
              else
                expect(json['data']['attributes'][param.to_s]).to eq(get_typed_value(type))
              end
            end
          end
        end

        # post with housing_request_ids:
        describe 'post request with housing_request_ids' do
          let!(:applicant2) {create(:applicant)}
          let!(:housing_request) {create(:housing_request, main_applicant: applicant2, is_active: true)}
          let!(:housing_request2) {create(:housing_request, main_applicant: applicant2)}
          before :each do
            authenticated_header(request, responsible)
            post 'create', {
                data: {
                    attributes: {
                        housing_request_ids: [housing_request.id.to_s, housing_request2.id.to_s],
                        first_name: 'Michel',
                        last_name: "Dupont",
                        email: "michel.dupont@gmail.com"
                    }
                }
            }
          end
          it 'returns status code 201 and the value appears in json' do
            expect(response).to have_http_status(201)
            #TODO on pourrait s'attendre a retrouver les ids des housings requests mais c'est coherent car elles sont deja en belongs to avec un autre applicant
            expect(json['data']['attributes']['housing_request_ids']).to eq([])
          end

          describe 'post request with procurating user' do
            let!(:applicant2) {create(:applicant)}
            let!(:housing_request) {create(:housing_request, main_applicant: applicant2, is_active: true)}
            let!(:housing_request2) {create(:housing_request, main_applicant: applicant2)}
            before :each do
              authenticated_header(request, responsible)
              post 'create', {
                  data: {
                      attributes: {
                          first_name: 'Michel',
                          last_name: "Dupont",
                          email: "michel.dupont@gmail.com"
                      }
                  }
              }
            end
            it 'returns status code 201 and the value appears in json' do
              expect(response).to have_http_status(201)
              #TODO on pourrait s'attendre a retrouver les ids des housings requests mais c'est coherent car elles sont deja en belongs to avec un autre applicant
              expect(json['data']['attributes']['first_name']).to eq('Michel')
              expect(json['data']['attributes']['readers']).to eq([responsible.id.to_s])
            end
          end
        end

        #post with entity_id
        describe 'post request with entity_ids' do
          let!(:applicant2) {create(:applicant)}
          let!(:entity) {create(:entity)}
          before :each do
            authenticated_header(request, responsible)
            post 'create', {
                data: {
                    attributes: {
                        entity_id: entity.id.to_s
                    }
                }
            }
          end
          it 'returns status code 201 and the value appears in json' do
            expect(response).to have_http_status(201)
            expect(json['data']['attributes']['entity_id']).to eq(entity.id.to_s)
          end
        end
      end

      describe 'GET /applicants' do
        describe 'GET /applicants' do
          let!(:applicant1) {create(:applicant, nationality: 'belge', apl_amount: 50)}
          let!(:applicant3) {create(:applicant, nationality: 'quebecquoise', apl_amount: 150)}
          let!(:applicant2) {create(:applicant, nationality: 'francaise', apl_amount: 150)}
          before :each do
            authenticated_header(request, responsible)
            get 'index'
          end
          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end
          it 'returns a valid json' do
            expect(json).not_to be_empty
            expect(json['data'].length).to eq(3)
            expect(json['meta']['pagination']).to be_present
            expect(json['meta']['pagination']['total_objects']).to eq(3)
          end
        end

        ACCEPTABLE_PARAMS.each do |acceptable_param|
          type = MUTATE_PARAMS[acceptable_param.to_sym] || MUTATE_PARAMS[acceptable_param.to_s]
          if type
            describe 'GET /applicants with sort' do
              let!(:applicant3) {create(:applicant, acceptable_param => get_typed_value(type, 2))}
              let!(:applicant1) {create(:applicant, acceptable_param => get_typed_value(type, 0))}
              let!(:applicant2) {create(:applicant, acceptable_param => get_typed_value(type, 1))}
              before :each do
                authenticated_header(request, responsible)
                get 'index', {sort: {acceptable_param => 1}}
              end
              it 'returns sorted results' do
                expect(response.status).to eq(200)
                if type == 'DateTime'
                  applicant_attribute = json['data'].map {|unit| DateTime.parse(unit['attributes'][acceptable_param])}
                  expect(applicant_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
                elsif type == 'Date'
                  applicant_attribute = json['data'].map {|unit| Date.parse(unit['attributes'][acceptable_param])}
                  expect(applicant_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
                else
                  applicant_attribute = json['data'].map {|unit| unit['attributes'][acceptable_param]}
                  expect(applicant_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
                end
              end
            end
          end
        end

        describe 'GET /applicants with operators' do
          let!(:applicant1) {create(:applicant, nationality: 'belge', apl_amount: 50)}
          let!(:applicant3) {create(:applicant, nationality: 'quebecquoise', apl_amount: 150)}
          let!(:applicant2) {create(:applicant, nationality: 'Samoan', apl_amount: 150)}
          describe 'with $lt' do
            before :each do
              authenticated_header(request, responsible)
              get 'index', {apl_amount: {'$lt': 100}}
            end
            it 'returns results' do
              expect(response.status).to eq(200)
              expect(json['meta']['pagination']['total_objects']).to eq(1)
            end
          end
          describe 'with $in' do
            before :each do
              authenticated_header(request, responsible)
              get 'index', {nationality: {'$in': ['belge', 'flamand']}}
            end
            it 'returns results' do
              expect(response.status).to eq(200)
              expect(json['meta']['pagination']['total_objects']).to eq(1)
            end
          end
          describe 'with $ne' do
            before :each do
              authenticated_header(request, responsible)
              get 'index', {apl_amount: {'$ne': 50}}
            end
            it 'returns sorted results' do
              expect(response.status).to eq(200)
              expect(json['meta']['pagination']['total_objects']).to eq(2)
            end
          end
          describe 'with $regex' do
            before :each do
              authenticated_header(request, responsible)
              get 'index', {nationality: {'$regex': /^S/}}
            end
            it 'returns sorted results' do
              expect(response.status).to eq(200)
              expect(json['meta']['pagination']['total_objects']).to eq(1)
            end
          end
        end
      end

      describe 'GET /applicants/:id' do
        describe 'verif it work' do
          let!(:applicant1) {create(:applicant, nationality: 'belge', apl_amount: 50)}
          before :each do
            authenticated_header(request, responsible)
            get 'show', {id: applicant1.id}
          end
          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end
          it 'returns a valid json' do
            expect(json).not_to be_empty
          end
        end
        describe 'verif it work not work with a non authorized user' do
          let!(:applicant1) {create(:applicant, nationality: 'belge', apl_amount: 50)}

          before :each do
            authenticated_header(request, user)
            get 'show', {id: applicant1.id}
          end
          it 'returns status code 404' do
            expect(response).to have_http_status(404)
          end
        end
      end

      describe 'PATCH /applicant/:id for year_minus_two' do
        let!(:demandeur_test) { create(:user, auth_roles: ['Demandeur']) }
        let!(:applicant_test) do
          create(
            :applicant,
            amount_year_minus_two: 1200,
            main_user_id: demandeur_test.id,
            user_id: demandeur_test.id,
            monthly_incomes: {
              'SAL': 1234
            }
          )
        end

        let!(:request_test) do
          create(
            :housing_request,
            is_active: true,
            main_applicant: applicant_test
          )
        end

        let(:params) { { amount_year_minus_two: 1201.0 } }

        before :each do
          authenticated_header(request, demandeur_test)
          patch 'update', {
            id: applicant_test.id.to_s,
            data: {
              attributes: params
            }
          }
        end

        it 'returns 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid json' do
          data = json['data']
          attributes = data['attributes']
          expect(data['id']).to eq applicant_test.id.to_s
          expect(attributes['eligibility_query']).to_not eq "{is_public:true}"
          expect(attributes['amount_year_minus_two']).to be params[:amount_year_minus_two]
          expect(attributes['eligibility_query'].json_to_h[:ceil_category_1]['$gte']).to be 1200.0
        end

        describe 'GET same object' do
          before :each do
            authenticated_header(request, demandeur_test)
            get 'show', id: applicant_test.id.to_s
          end

          it 'returns 200' do
            expect(response).to have_http_status(200)
          end

          it 'returns valid json' do
            data = json['data']
            attributes = data['attributes']
            expect(data['id']).to eq applicant_test.id.to_s
            expect(attributes['eligibility_query']).to_not eq "{is_public:true}"
            expect(attributes['amount_year_minus_two']).to be params[:amount_year_minus_two]
            expect(attributes['eligibility_query'].json_to_h[:ceil_category_1]['$gte']).to be params[:amount_year_minus_two]
          end
        end
      end

      describe 'PATCH /applicants/:id' do
        applicant_params = {}
        MUTATE_PARAMS.each do |param, type|
          applicant_params[param] = get_typed_value(type)
        end
        let!(:applicant) {create(:applicant, **applicant_params)}
        MUTATE_PARAMS.each do |param, type|
          describe 'patch request with ' + param.to_s + ' ' + type.to_s + 'is valid' do
            let(:params) do
              attr = {}
              attr[param] = get_typed_value(type, 2)
              attr
            end
            before :each do
              authenticated_header(request, responsible)
              patch 'update', {
                  id: applicant.id.to_s,
                  data: {
                      attributes: params
                  }
              }
            end
            it 'returns status code 200 and the value appears in json' do
              expect(response).to have_http_status(200)
              if (type == 'Date' || type == 'DateTime')

                expect(json['data']['attributes'][param.to_s]).not_to be_empty
                expect(json['data']['attributes'][param.to_s]).not_to eq('')
              else
                expect(json['data']['attributes'][param.to_s]).to eq(get_typed_value(type, 2))
              end
            end
          end
        end
        describe 'put'do
          let!(:applicant3) {create(:applicant,first_name: 'Jac', last_name: "Mike", full_name: "Jackie Mike",email: "jackie.mike@gmail.com", writers:[responsible.id], status: "OK", monthly_incomes: {porn: 30}, monthly_charges: {cat_food: 10})}

          before :each do
            authenticated_header(request, responsible)
            patch 'update', {
                id: applicant3.id.to_s,
                data: {
                    attributes: {

                        sms_notification: true
                    }
                }
            }
          end
          it 'returns status code 201 and the value appears in json' do
            expect(response).to have_http_status(200)
           expect(json['data']['attributes']['first_name']).to eq("Jac")
          end
        end
        describe 'put change user of a'do
          let!(:applicant4) {create(:applicant, main_user_id: responsible.id.to_s, email: "jackie.mike@gmail.com", status: "OK", monthly_incomes: {porn: 30}, monthly_charges: {cat_food: 10})}

          before :each do
            applicant4.writers.push(responsible.id.to_s)
            applicant4.save
            authenticated_header(request, responsible)
            put 'update', {
              id: applicant4.id.to_s,
              data: {
                attributes: {
                  main_user_id: user.id,
                  sms_notification: true
                }
              }
            }
          end
          it 'returns status code 201 and the value appears in json' do
            expect(response).to have_http_status(200)
            expect(json['data']['attributes']['first_name']).to eq("Jackie")
            expect(json['data']['attributes']['writers']).to_not include(responsible.id.to_s)
            expect(json['data']['attributes']['writers']).to include(user.id.to_s)
          end
        end

        #post with entity_id
        describe 'patch request with entity_ids' do
          let!(:entity) {create(:entity)}
          before :each do
            authenticated_header(request, responsible)

            patch 'update', {
                id: applicant.id.to_s,
                data: {
                    attributes: {
                        entity_id: entity.id.to_s
                    }
                }
            }
          end
          it 'returns status code 200 and the value appears in json' do
            expect(response).to have_http_status(200)
            expect(json['data']['attributes']['entity_id']).to eq(entity.id.to_s)
          end
        end
      end

      describe 'PATCH /applicants/:id' do
        #TODO : est ce que c'est nécessaire ?
      end

      describe 'DELETE /applicants/:id' do
        let!(:applicant) {create(:applicant, user_id: responsible.id.to_s)}
        before :each do
          authenticated_header(request, responsible)

          delete 'destroy', {
              id: applicant.id.to_s
          }
        end
        it 'returns status code 404' do
          expect(response).to have_http_status(204)
        end
      end
    end

    context 'with bear minimum for eligibility checking' do
      let!(:admin) { create(:administrator) }

      let!(:threshold1) do
        create(
            :threshold,
            type: 'PLA LM',
            thresholds_map: {
                'Paris et communes limitrophes': {
                    'thresholds_list': [
                        13_050,
                        21_272,
                        27_883,
                        30_521,
                        36_307,
                        40_859
                    ],
                    'another_person': 4552
                },
                'IDF hors Paris et Communes limitrophes': {
                    'thresholds_list': [
                        13_050,
                        21_272,
                        25_569,
                        28_075,
                        33_238,
                        37_401
                    ],
                    'another_person': 4166
                },
                'Autres régions': {
                    'thresholds_list': [
                        11_342,
                        16_525,
                        19_872,
                        22_111,
                        25_870,
                        29_155
                    ],
                    'another_person': 3252
                }
            }
        )
      end

      let!(:user1) {create(:user)}
      let!(:applicant1) do
        create(
            :applicant,
            sne_profile_creation_date: '2016-09-24',
            user_id: user1.id,
            amount_year_minus_two: 9787.24,
            monthly_incomes: {
                'Salaire': 1045.23,
                'APL': 321.62
            }
        )
      end
      let!(:request1) do
        create(
          :housing_request,
          is_active: true,
          main_applicant: applicant1,
          seeked_departments: ['34']
        )
      end

      let!(:user2) {create(:user)}
      let!(:applicant2) do
        create(
          :applicant,
          sne_profile_creation_date: '2017-09-24',
          user_id: user2.id,
          amount_year_minus_two: 9787.24,
          monthly_incomes: {
            'Salaire': 1045.23,
            'APL': 321.62
          }
        )
      end
      let!(:request2) do
        create(
          :housing_request,
          is_active: true,
          main_applicant: applicant2,
          seeked_departments: ['34']
        )
      end

      let(:group1) { create(:group, user_ids: [user1.id]) }

      let!(:offer1) do
        create(
          :housing_offer,
          rent_amount: 234.32,
          rental_charges: 42.15,
          parking_charges: 12.71,
          surface: 29,
          typology: 'T2',
          threshold_type: 'PLA LM',
          offer_status: 'LOGEMENT PUBLIE',
          department: '34',
          threshold_subtype: 'IDF hors Paris et Communes limitrophes',
          min_effort_rate: 0,
          readers: ["g_#{group1.id}"]
        )
      end

      let!(:offer2) do
        create(
          :housing_offer,
          rent_amount: 434.32,
          rental_charges: 53.12,
          parking_charges: 22.43,
          surface: 34,
          typology: 'T2',
          threshold_type: 'PLA LM',
          offer_status: 'LOGEMENT PUBLIE',
          department: '34',
          threshold_subtype: 'Autres régions',
          min_effort_rate: 0,
          readers: ["g_#{group1.id}"]
        )
      end

      describe 'GET /applicants/:id/eligible_offers' do
        before :each do
          perform_enqueued_jobs do
            enqueued_jobs.each do |job|
              if job[:job].to_s == "UpdateScoreJob"
                UpdateScoreJob.perform_now(*job[:args])
              end
            end
          end
          authenticated_header(request, admin)
          get 'eligible_offers', id: applicant1.id
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid JSON' do
          data = json['data']
          expect(data.length).to eq(2)
          first_attributes = data[0]['attributes']
          expect(
            data.map do |line|
              [line['attributes']['eligible_position'], line['attributes']['profile_epci_score']]
            end
          ).to all(all(be_a Integer))
        end
      end

      describe 'GET /applicants/:id/eligible_offers with sort on profile_epci_score' do
        before :each do
          perform_enqueued_jobs do
            enqueued_jobs.each do |job|
              if job[:job].to_s == "UpdateScoreJob"
                UpdateScoreJob.perform_now(*job[:args])
              end
            end
          end
          authenticated_header(request, admin)
          get 'eligible_offers', id: applicant1.id, sort: { 'profile_epci_score' => -1 }
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid JSON' do
          expect(Hermes::Score.where(applicant_id: applicant1.id, is_active: true).count).to be 2
          data = json['data']
          expect(
            data.map { |elt| elt['attributes']['profile_epci_score'] }.each_cons(2).all? { |a, b| b <= a }
          ).to be true
          expect(data.length).to be 2
        end
      end

      describe 'GET /applicants/:id/eligible_offers with sort on profile_epci_score' do
        before :each do
          perform_enqueued_jobs do
            enqueued_jobs.each do |job|
              if job[:job].to_s == "UpdateScoreJob"
                UpdateScoreJob.perform_now(*job[:args])
              end
            end
          end
          authenticated_header(request, admin)
          get 'eligible_offers', id: applicant1.id, profile_epci_score: { '$gt' => 0 }
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid JSON' do
          expect(Hermes::Score.where(applicant_id: applicant1.id, is_active: true).count).to be 2
          data = json['data']
          expect(
            data.first['attributes']['profile_epci_score']
          ).to be > 0
          expect(data.length).to be 1
        end
      end
    end
  end

  describe Api::Dmo::ApplicantsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    TAGS_API_DMO_APPLICANTS= 'API_DMO_APPLICANTS'
    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes
    let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
    let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}

    context 'as admin' do
      let!(:admin) {create(:administrator)}

      path '/api/dmo/applicants' do
        post 'Create applicant' do
          tags TAGS_API_DMO_APPLICANTS
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key]= {type: nice_type(val)}
          end
          json_params[:housing_request_ids]={type: :array}
          json_params[:entity_ids]={type: :array}
          parameter(
            name: :params,
            in: :body,
            schema: jsonify_params(json_params),
            description: '<span>' \
              '<div class="opblock opblock-delete" style="padding: 4px; align-items: center; display: flex">' \
              '<span class="opblock-summary-method" style="margin-right: 4px">WARNING</span>' \
              '<span class="no-style">Setting `main_employer_siret, will ' \
              ' change its relationship with the corresponding entity.</span></div></span>'
          )
          #jsonapi_params( **json_params )

          response '201', 'applicant created' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:params) do
              jsonapi_attributes(
                  user_id: responsible.id.to_s
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'applicants'
            end
          end
        end
      end

      path '/api/dmo/applicants' do
        get 'Retrieves all applicants' do
          tags TAGS_API_DMO_APPLICANTS
          produces 'application/json'

          parameter name: 'ids[]', in: :query, type: :array, items: { type: :string }, collectionFormat: :multi, required: false
          ACCEPTABLE_PARAMS.each do |acceptable_param|
            type = :string
            if MUTATE_PARAMS[acceptable_param]
              type = nice_type(MUTATE_PARAMS[acceptable_param] )
            end
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end
          #VALID_OPERATORS.each do |valid_operator|
           # parameter( name: valid_operator.to_s, in: :query,type: :object, required: false)
   #      ## end
          security [Bearer: {}]
          paginator

          response '200', 'applicants found' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let!(:applicants) { create_list(:applicant, 5,user_id: responsible.id.to_s) }

            include_context 'with integration test'
            include_context 'valid jsonapi collection' do
              let(:expected_collection) { applicants }
            end
          end
        end
      end

      path '/api/dmo/applicants/{id}/eligible_offers' do
        get 'Retrieves eligible offers for a given applicant' do
          tags TAGS_API_DMO_APPLICANTS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string,  required: true
          parameter name: 'ids[]', in: :query, type: :array, items: { type: :string }, collectionFormat: :multi, required: false
          security [Bearer: {}]
          OFFERS_ACCEPTABLE_PARAMS.each do |acceptable_param|
            parameter name: acceptable_param.to_s, in: :query,  required: false
          end
          paginator

          response '200', 'housing_offers found' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:applicant) { create(:applicant,user_id: responsible.id.to_s) }
            let(:id) { applicant.id }

            include_context 'not containings errors'
            run_test!

          end
        end
      end

      path '/api/dmo/applicants/{id}' do
        get 'Retrieves a single applicant by id' do
          tags TAGS_API_DMO_APPLICANTS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '200', 'applicant found' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:applicant) { create(:applicant) }
            let(:id) { applicant.id }

            include_context 'with integration test'
            include_context 'valid jsonapi ressource' do
              let(:expected_ressource) { applicant }
            end
          end
        end
      end

      path '/api/dmo/applicants/{id}' do
        put 'edit applicant' do
          tags TAGS_API_DMO_APPLICANTS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key]= {type: nice_type(val)}
          end
          json_params[:main_applicant_id]={type: :string}
          json_params[:coapplicants]={type: :array}
          json_params[:housing_application_ids]={type: :array}
          json_params[:applicant_ids]={type: :array}
          json_params[:reference_tax_revenues]={type: :float}
          json_params[:seeked_departments]={type: :array}
          json_params[:dependents]={type: :array}
          parameter(
            name: :params,
            in: :body,
            schema: jsonify_params(json_params),
            description:  '<span>' \
            '<div class="opblock opblock-delete" style="padding: 4px; align-items: center; display: flex">' \
            '<span class="opblock-summary-method" style="margin-right: 4px">WARNING</span>' \
            '<span class="no-style">Setting `main_employer_siret, will ' \
            ' change its relationship with the corresponding entity.</span></div></span>'
          )
          #jsonapi_params( **json_params )

          response '200', 'applicant created' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:applicant) {create(:applicant, user_id: responsible.id.to_s)}
            let(:id) {applicant.id}
            let(:params) do
              jsonapi_attributes(
                  id: id,
                  first_name: 'Joseph',
                  last_name: 'Joestar'
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              data = json['data']
              expect(data).to be_a Hash
              expect(data['id']).to_not be nil
              attributes = data['attributes']
              expect(attributes['first_name']).to eq 'Joseph'
              expect(attributes['last_name']).to eq 'Joestar'
            end
          end
        end
      end

      path '/api/dmo/applicants/{id}' do
        delete 'Delete a single applicant by id' do
          tags TAGS_API_DMO_APPLICANTS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '204', 'applicant deleted' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:id) { create(:applicant, responsible: admin.id).id }
            run_test!
          end
        end
      end
    end
  end
end
